@@ -80,7 +80,7 @@ static BOOL read_backslash(char**p, sBuf* buf, char* sname, int* sline)
 
     switch(**p) {
         case 0:
-            err_msg("unexpected end. can't quote null.", sname, *sline, buf->mBuf);
+            err_msg("unexpected end. can't quote null.", sname, *sline);
             return FALSE;
 
         case '\n':
@@ -205,7 +205,7 @@ static BOOL read_dollar(char** p, sBuf* buf, char* sname, int* sline, sCommand*
 
             while(1) {
                 if(**p == 0) {
-                    err_msg("close } which is used by expand variable", sname, *sline, name.mBuf);
+                    err_msg("close } which is used by expand variable", sname, *sline);
                     FREE(name.mBuf);
                     FREE(key.mBuf);
                     FREE(init_value.mBuf);
@@ -262,7 +262,7 @@ static BOOL read_dollar(char** p, sBuf* buf, char* sname, int* sline, sCommand*
 
             skip_spaces(p);
             if(**p != ']') {
-                err_msg("require ] character", sname, *sline, name.mBuf);
+                err_msg("require ] character", sname, *sline);
                 FREE(name.mBuf);
                 FREE(key.mBuf);
                 FREE(init_value.mBuf);
@@ -318,24 +318,47 @@ static BOOL read_backquote(char** p, sBuf* buf, char* sname, int* sline, sComman
     return TRUE;
 }
 
-static BOOL colon_statment(char** p, ALLOC sNodeTree** node, char* sname, int* sline)
+static BOOL colon_statment(char** p, sStatment* statment, char* sname, int* sline)
 {
-    *node = NULL;
+    statment->mNodeTreeNum = 0;
     while(**p) {
         if(**p == ')' || **p == ']' || **p == '`' || **p == '}') {
             break;
         }
-        if(**p == '\n') {
+        else if(**p == '\n') {
+            (*sline)++;
+            (*p)++;
+            break;
+        }
+        else if(**p == ';') {
             (*p)++;
             break;
         }
         else {
-            if(!expression_add_sub(ALLOC node, p, sname, sline)) {
-                sNodeTree_free(*node);
+            sNodeTree* node;
+            if(!node_expression(ALLOC &node, p, sname, sline)) {
+                sNodeTree_free(node);
                 return FALSE;
             }
+            statment->mNodeTree[statment->mNodeTreeNum++] = MANAGED node;
+
+            if(statment->mNodeTreeNum == STATMENT_NODE_MAX) {
+                err_msg("overflow number of node tree", sname, *sline);
+                return FALSE;
+            }
+
+            skip_spaces(p);
+
+            if(**p == ',') {
+                (*p)++;
+                skip_spaces(p);
+            }
+            else {
+                break;
+            }
         }
     }
+
     return TRUE;
 }
 
@@ -360,7 +383,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
         }
         else if(squote || dquote) {
             if(**p == 0) {
-                err_msg("close single quote or double quote", sname, *sline, buf->mBuf);
+                err_msg("close single quote or double quote", sname, *sline);
                 return FALSE;
             }
             else if(dquote) {
@@ -419,7 +442,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
                         break;
                     }
                     else if(**p == 0) {
-                        err_msg("reqire to close %q quote",  sname, *sline, buf->mBuf);
+                        err_msg("reqire to close %q quote",  sname, *sline);
                         return FALSE;
                     }
                     else if(**p == '\\') {
@@ -447,7 +470,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
                         break;
                     }
                     else if(**p == 0) {
-                        err_msg("reqire to close %q quote",  sname, *sline, buf->mBuf);
+                        err_msg("reqire to close %q quote",  sname, *sline);
                         return FALSE;
                     }
                     else {
@@ -525,7 +548,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
                     (*p)++;
                 }
                 else {
-                    err_msg("invalid here document name", sname, *sline, buf->mBuf);
+                    err_msg("invalid here document name", sname, *sline);
                     return FALSE;
                 }
             }
@@ -537,19 +560,19 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
                 (*p)++;
             }
             else {
-                err_msg("invalid here document name", sname, *sline, buf->mBuf);
+                err_msg("invalid here document name", sname, *sline);
                 return FALSE;
             }
 
             if(string_c_str(name)[0] == 0) {
-                err_msg("invalid here document name", sname, *sline, buf->mBuf);
+                err_msg("invalid here document name", sname, *sline);
                 return FALSE;
             }
 
             sObject* line = STRING_NEW_STACK("");
             while(1) {
                 if(**p == 0) {
-                    err_msg("require to close Here Document", sname, *sline, "here document");
+                    err_msg("require to close Here Document", sname, *sline);
                     return FALSE;
                 }
                 /// env ///
@@ -559,7 +582,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
                     }
                 }
                 else if(gXyzshSigInt) {
-                    err_msg("signal interrupt", sname, *sline, "here document");
+                    err_msg("signal interrupt", sname, *sline);
                     return FALSE;
                 }
                 else if(**p == '\n') {
@@ -678,7 +701,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
             buf->mGlob = TRUE;
 
             if(buf->mOption) {
-                err_msg("can't take option with glob", sname, *sline, buf->mBuf);
+                err_msg("can't take option with glob", sname, *sline);
                 return FALSE;
             }
         }
@@ -691,7 +714,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
             buf->mOption = TRUE;
 
             if(buf->mGlob) {
-                err_msg("can't take option with glob", sname, *sline, buf->mBuf);
+                err_msg("can't take option with glob", sname, *sline);
                 return FALSE;
             }
         }
@@ -718,7 +741,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
     }
 
     if(squote || dquote) {
-        err_msg("require to close ' or \"", sname, *sline, buf->mBuf);
+        err_msg("require to close ' or \"", sname, *sline);
         return FALSE;
     }
 
@@ -814,14 +837,14 @@ static BOOL block_parse(char** p, char* sname, int* sline, sObject* block, sObje
         while(**p == ' ' || **p == '\t' || **p == '\n' && (*sline)++) { (*p)++; }
 
         /// there is no command, delete last statment
-        if(statment->mCommandsNum == 0) {
+        if(statment->mCommandsNum == 0 && (statment->mFlags & STATMENT_FLAGS_KIND_NODETREE) == 0) {
             sStatment_delete(statment);
             memset(statment, 0, sizeof(sStatment));
             SBLOCK(block).mStatmentsNum--;
         }
 
         if(**p == 0) {
-            err_msg("close block. require ) character before the end of statments", sname, *sline, NULL);
+            err_msg("close block. require ) character before the end of statments", sname, *sline);
             return FALSE;
         }
 
@@ -867,14 +890,14 @@ static BOOL block_parse_until_backquote(char** p, char* sname, int* sline, sObje
         while(**p == ' ' || **p == '\t' || **p == '\n' && (*sline)++) { (*p)++; }
 
         /// there is no command, delete last statment
-        if(statment->mCommandsNum == 0) {
+        if(statment->mCommandsNum == 0 && (statment->mFlags & STATMENT_FLAGS_KIND_NODETREE) == 0) {
             sStatment_delete(statment);
             memset(statment, 0, sizeof(sStatment));
             SBLOCK(block).mStatmentsNum--;
         }
 
         if(**p == 0) {
-            err_msg("close block. require ) character before the end of statments", sname, *sline, NULL);
+            err_msg("close block. require ) character before the end of statments", sname, *sline);
             return FALSE;
         }
 
@@ -906,7 +929,7 @@ static BOOL add_argument_to_command(MANAGED sBuf* buf, sCommand* command, char*
     /// redirect ///
     if(buf->mRedirect) {
         if(buf->mOption) {
-            err_msg("can't take redirect with option", sname, *sline, NULL);
+            err_msg("can't take redirect with option", sname, *sline);
             return FALSE;
         }
         if(!sCommand_add_redirect(command, MANAGED buf->mBuf, buf->mEnv, buf->mQuotedString, buf->mGlob, buf->mRedirect, sname, *sline)) {
@@ -1046,19 +1069,25 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
 /// FALSE: indicate to occure error
 static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object, BOOL read_until_backquote)
 {
-/*
     /// expression ///
     if(**p == ':' && *(*p+1) != ':') {
-        p++;
+        (*p)++;
         skip_spaces(p);
 
-        sNodeTree* node;
-        if(!colon_statment(p, ALLOC &node, sname, sline)) {
+        if(**p == '@') {
+            statment->mFlags |= STATMENT_FLAGS_NODETREE_OUTPUT;
+            (*p)++;
+            skip_spaces(p);
+        }
+
+        statment->mFlags |= STATMENT_FLAGS_KIND_NODETREE;
+
+        if(!colon_statment(p, statment, sname, sline)) {
             return FALSE;
         }
     }
+    /// normal pipe statment ///
     else {
-*/
         /// a reversing code ///
         if(**p == '!') {
             (*p)++;
@@ -1098,7 +1127,7 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
 
         while(**p) {
             if(statment->mCommandsNum >= STATMENT_COMMANDS_MAX) {
-                err_msg("Overflow command number. Please use global pipe.", sname, *sline, NULL);
+                err_msg("Overflow command number. Please use global pipe.", sname, *sline);
                 return FALSE;
             }
 
@@ -1116,7 +1145,7 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
             if(command->mArgsNum == 0)
             {
                 if(command->mMessagesNum > 0) {
-                    err_msg("require command name", sname, *sline, NULL);
+                    err_msg("require command name", sname, *sline);
                     return FALSE;
                 }
                 else {
@@ -1177,7 +1206,7 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
             skip_spaces(p);
 
             if(gAppType == kATOptC) {
-                err_msg("In script mode a job can't run on background. On interactive shell a job can run on background", sname, *sline, NULL);
+                err_msg("In script mode a job can't run on background. On interactive shell a job can run on background", sname, *sline);
                 return FALSE;
             }
 
@@ -1207,10 +1236,10 @@ static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sn
         else {
             char buf[128];
             snprintf(buf, 128, "unexpected token3 -->(%c)\n", **p);
-            err_msg(buf, sname, *sline, NULL);
+            err_msg(buf, sname, *sline);
             return FALSE;
         }
-//    }
+    }
 
     return TRUE;
 }
@@ -1237,7 +1266,7 @@ BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_o
         }
 
         /// There is not a command, so delete last statment in block
-        if(statment->mCommandsNum == 0) {
+        if(statment->mCommandsNum == 0 && (statment->mFlags & STATMENT_FLAGS_KIND_NODETREE) == 0) {
             sStatment_delete(statment);
             memset(statment, 0, sizeof(sStatment));
             SBLOCK(block).mStatmentsNum--;
@@ -1249,7 +1278,7 @@ BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_o
         if(*p == ')') {
             char buf[128];
             snprintf(buf, 128, "unexpected token3 -->(%c)\n", *p);
-            err_msg(buf, sname, *sline, NULL);
+            err_msg(buf, sname, *sline);
             return FALSE;
         }
         /// skip comment ///
